# flake8: NOQA
from .details import DossierDetail, DossierDetailType, DossierDetailsMixin
from .accommodation_dossier import AccommodationDossier
from .activity_dossier import ActivityDossier
from .place_dossier import PlaceDossier
from .transport_dossier import TransportDossier
